@@ -215,7 +215,14 @@
 #define ESR_FSC_PERMF_L1	U(0x0d)
 #define ESR_FSC_PERMF_L2	U(0x0e)
 #define ESR_FSC_PERMF_L3	U(0x0f)
+#define ESR_FSC_SEA_NTT		U(0x10)
 #define ESR_FSC_TAG_CHECK	U(0x11)
+#define ESR_FSC_SEA_TT_SUB_L2	U(0x12)
+#define ESR_FSC_SEA_TT_SUB_L1	U(0x13)
+#define ESR_FSC_SEA_TT_L0	U(0x14)
+#define ESR_FSC_SEA_TT_L1	U(0x15)
+#define ESR_FSC_SEA_TT_L2	U(0x16)
+#define ESR_FSC_SEA_TT_L3	U(0x17)
 #define ESR_FSC_ALIGN		U(0x21)
 
 /* WnR for DABT and RES0 for IABT */
@@ -471,6 +478,12 @@ DEFINE_U64_REG_READ_FUNC(sctlr_el1)
 DEFINE_REG_READ_FUNC_(cntfrq, uint32_t, cntfrq_el0)
 DEFINE_REG_READ_FUNC_(cntvct, uint64_t, cntvct_el0)
 DEFINE_REG_READ_FUNC_(cntpct, uint64_t, cntpct_el0)
+DEFINE_REG_READ_FUNC_(cntp_ctl, uint32_t, cntp_ctl_el0)
+DEFINE_REG_WRITE_FUNC_(cntp_ctl, uint32_t, cntp_ctl_el0)
+DEFINE_REG_READ_FUNC_(cntp_tval, uint32_t, cntp_tval_el0)
+DEFINE_REG_WRITE_FUNC_(cntp_tval, uint32_t, cntp_tval_el0)
+DEFINE_REG_READ_FUNC_(cntp_cval, uint64_t, cntp_cval_el0)
+DEFINE_REG_WRITE_FUNC_(cntp_cval, uint64_t, cntp_cval_el0)
 DEFINE_REG_READ_FUNC_(cntkctl, uint32_t, cntkctl_el1)
 DEFINE_REG_WRITE_FUNC_(cntkctl, uint32_t, cntkctl_el1)
 DEFINE_REG_READ_FUNC_(cntps_ctl, uint32_t, cntps_ctl_el1)
