@@ -26,18 +26,21 @@
 #define FFA_NO_DATA		-9
 
 /* FFA_VERSION helpers */
-#define FFA_VERSION_MAJOR		U(1)
 #define FFA_VERSION_MAJOR_SHIFT		U(16)
 #define FFA_VERSION_MAJOR_MASK		U(0x7FFF)
-#define FFA_VERSION_MINOR		U(1)
 #define FFA_VERSION_MINOR_SHIFT		U(0)
 #define FFA_VERSION_MINOR_MASK		U(0xFFFF)
 #define MAKE_FFA_VERSION(major, minor)	\
 	((((major) & FFA_VERSION_MAJOR_MASK) << FFA_VERSION_MAJOR_SHIFT) | \
 	 ((minor) & FFA_VERSION_MINOR_MASK))
+#define FFA_GET_MAJOR_VERSION(vers)	\
+	(((uint32_t)(vers) >> FFA_VERSION_MAJOR_SHIFT) & FFA_VERSION_MAJOR_MASK)
+#define FFA_GET_MINOR_VERSION(vers)	((uint32_t)(vers) & \
+					 FFA_VERSION_MINOR_MASK)
 
 #define FFA_VERSION_1_0			MAKE_FFA_VERSION(1, 0)
 #define FFA_VERSION_1_1			MAKE_FFA_VERSION(1, 1)
+#define FFA_VERSION_1_2			MAKE_FFA_VERSION(1, 2)
 
 /* Function IDs */
 #define FFA_ERROR			U(0x84000060)
@@ -47,6 +50,7 @@
 #define FFA_VERSION			U(0x84000063)
 #define FFA_FEATURES			U(0x84000064)
 #define FFA_RX_RELEASE			U(0x84000065)
+#define FFA_RX_ACQUIRE			U(0x84000084)
 #define FFA_RXTX_MAP_32			U(0x84000066)
 #define FFA_RXTX_MAP_64			U(0xC4000066)
 #define FFA_RXTX_UNMAP			U(0x84000067)
@@ -62,6 +66,8 @@
 #define FFA_MSG_SEND_DIRECT_REQ_64	U(0xC400006F)
 #define FFA_MSG_SEND_DIRECT_RESP_32	U(0x84000070)
 #define FFA_MSG_SEND_DIRECT_RESP_64	U(0xC4000070)
+#define FFA_MSG_SEND_DIRECT_REQ2	U(0xC400008D)
+#define FFA_MSG_SEND_DIRECT_RESP2	U(0xC400008E)
 #define FFA_MSG_POLL			U(0x8400006A)
 #define FFA_MEM_DONATE_32		U(0x84000071)
 #define FFA_MEM_DONATE_64		U(0xC4000071)
@@ -92,6 +98,8 @@
 #define FFA_MEM_PERM_SET_64		U(0xC4000089)
 #define FFA_CONSOLE_LOG_32		U(0x8400008A)
 #define FFA_CONSOLE_LOG_64		U(0xC400008A)
+#define FFA_EL3_INTR_HANDLE		U(0x8400008C)
+#define FFA_PARTITION_INFO_GET_REGS	U(0xC400008B)
 
 #define FFA_FEATURES_FUNC_ID_MASK	BIT32(31)
 #define FFA_FEATURES_FEATURE_ID_MASK	GENMASK_32(7, 0)
@@ -231,11 +239,13 @@
 #define FFA_BOOT_INFO_FLAG_CONTENT_FORMAT_ADDR	U(0)
 
 #define FFA_BOOT_INFO_SIGNATURE		U(0xFFA)
-#define FFA_BOOT_INFO_VERSION		U(0x10001)
+#define FFA_BOOT_INFO_VERSION_1_1	U(0x10001)
+#define FFA_BOOT_INFO_VERSION_1_2	U(0x10002)
 
-#define FFA_CONSOLE_LOG_CHAR_COUNT_MASK	GENMASK_32(7, 0)
-#define FFA_CONSOLE_LOG_32_MAX_MSG_LEN	U(24)
-#define FFA_CONSOLE_LOG_64_MAX_MSG_LEN	U(48)
+#define FFA_CONSOLE_LOG_CHAR_COUNT_MASK		GENMASK_32(7, 0)
+#define FFA_CONSOLE_LOG_32_MAX_MSG_LEN		U(24)
+#define FFA_CONSOLE_LOG_64_V1_1_MAX_MSG_LEN	U(48)
+#define FFA_CONSOLE_LOG_64_MAX_MSG_LEN		U(128)
 
 /* Memory transaction type in FFA_MEM_RETRIEVE_RESP flags */
 #define FFA_MEMORY_TRANSACTION_TYPE_MASK	GENMASK_32(4, 3)
