@@ -33,10 +33,12 @@
 #include <mm/core_memprot.h>
 #include <mm/core_mmu.h>
 #include <mm/fobj.h>
+#include <mm/page_alloc.h>
 #include <mm/phys_mem.h>
 #include <mm/tee_mm.h>
 #include <mm/tee_pager.h>
 #include <sm/psci.h>
+#include <stdalign.h>
 #include <trace.h>
 #include <utee_defines.h>
 #include <util.h>
@@ -87,7 +89,7 @@ DECLARE_KEEP_PAGER(sem_cpu_sync);
 vaddr_t boot_cached_mem_end __nex_data = 1;
 
 static unsigned long boot_arg_fdt __nex_bss;
-static unsigned long boot_arg_nsec_entry __nex_bss;
+unsigned long boot_arg_nsec_entry __nex_bss;
 static unsigned long boot_arg_pageable_part __nex_bss;
 static unsigned long boot_arg_transfer_list __nex_bss;
 static struct transfer_list_header *mapped_tl __nex_bss;
@@ -120,36 +122,6 @@ __weak unsigned long plat_get_aslr_seed(void)
 	return 0;
 }
 
-#if defined(_CFG_CORE_STACK_PROTECTOR) || defined(CFG_WITH_STACK_CANARIES)
-/* Generate random stack canary value on boot up */
-__weak void plat_get_random_stack_canaries(void *buf, size_t ncan, size_t size)
-{
-	TEE_Result ret = TEE_ERROR_GENERIC;
-	size_t i = 0;
-
-	assert(buf && ncan && size);
-
-	/*
-	 * With virtualization the RNG is not initialized in Nexus core.
-	 * Need to override with platform specific implementation.
-	 */
-	if (IS_ENABLED(CFG_NS_VIRTUALIZATION)) {
-		IMSG("WARNING: Using fixed value for stack canary");
-		memset(buf, 0xab, ncan * size);
-		goto out;
-	}
-
-	ret = crypto_rng_read(buf, ncan * size);
-	if (ret != TEE_SUCCESS)
-		panic("Failed to generate random stack canary");
-
-out:
-	/* Leave null byte in canary to prevent string base exploit */
-	for (i = 0; i < ncan; i++)
-		*((uint8_t *)buf + size * i) = 0;
-}
-#endif /* _CFG_CORE_STACK_PROTECTOR || CFG_WITH_STACK_CANARIES */
-
 /*
  * This function is called as a guard after each smc call which is not
  * supposed to return.
@@ -354,6 +326,7 @@ static void init_asan(void)
 	 * Add access to areas that aren't opened automatically by a
 	 * constructor.
 	 */
+	boot_mem_init_asan();
 	asan_tag_access(&__ctor_list, &__ctor_end);
 	asan_tag_access(__rodata_start, __rodata_end);
 #ifdef CFG_WITH_PAGER
@@ -895,11 +868,20 @@ void init_tee_runtime(void)
 		thread_update_canaries();
 }
 
-static void init_primary(unsigned long pageable_part, unsigned long nsec_entry)
+static bool add_padding_to_pool(vaddr_t va, size_t len, void *ptr __unused)
+{
+#ifdef CFG_NS_VIRTUALIZATION
+	nex_malloc_add_pool((void *)va, len);
+#else
+	malloc_add_pool((void *)va, len);
+#endif
+	return true;
+}
+
+static void init_primary(unsigned long pageable_part)
 {
 	vaddr_t va = 0;
 
-	thread_init_core_local_stacks();
 	/*
 	 * Mask asynchronous exceptions before switch to the thread vector
 	 * as the thread handler requires those to be masked while
@@ -934,9 +916,21 @@ static void init_primary(unsigned long pageable_part, unsigned long nsec_entry)
 	malloc_add_pool(__heap1_start, __heap1_end - __heap1_start);
 #endif
 	IMSG_RAW("\n");
+	if (IS_ENABLED(CFG_DYN_CONFIG)) {
+		size_t sz = sizeof(struct thread_core_local) *
+			    CFG_TEE_CORE_NB_CORE;
+		void *p = boot_mem_alloc(sz, alignof(void *) * 2);
+
+#ifdef CFG_NS_VIRTUALIZATION
+		nex_malloc_add_pool(p, sz);
+#else
+		malloc_add_pool(p, sz);
+#endif
+	}
 
 	core_mmu_save_mem_map();
 	core_mmu_init_phys_mem();
+	boot_mem_foreach_padding(add_padding_to_pool, NULL);
 	va = boot_mem_release_unused();
 	if (!IS_ENABLED(CFG_WITH_PAGER)) {
 		/*
@@ -947,6 +941,17 @@ static void init_primary(unsigned long pageable_part, unsigned long nsec_entry)
 		boot_cached_mem_end = va;
 	}
 
+	if (IS_ENABLED(CFG_DYN_CONFIG)) {
+		/*
+		 * This is needed to enable virt_page_alloc() now that
+		 * boot_mem_alloc() can't be used any longer.
+		 */
+		if (IS_ENABLED(CFG_NS_VIRTUALIZATION))
+			nex_page_alloc_init();
+		else
+			page_alloc_init();
+	}
+
 	if (IS_ENABLED(CFG_WITH_PAGER)) {
 		/*
 		 * Pager: init_runtime() calls thread_kernel_enable_vfp()
@@ -959,20 +964,9 @@ static void init_primary(unsigned long pageable_part, unsigned long nsec_entry)
 		init_pager_runtime(pageable_part);
 	}
 
-	if (IS_ENABLED(CFG_NS_VIRTUALIZATION)) {
-		/*
-		 * Virtualization: We can't initialize threads right now because
-		 * threads belong to "tee" part and will be initialized
-		 * separately per each new virtual guest. So, we'll clear
-		 * "curr_thread" and call it done.
-		 */
-		thread_get_core_local()->curr_thread = -1;
-	} else {
-		thread_init_boot_thread();
-	}
-	thread_init_primary();
+	/* Initialize canaries around the stacks */
+	thread_init_canaries();
 	thread_init_per_cpu();
-	init_sec_mon(nsec_entry);
 }
 
 static bool cpu_nmfi_enabled(void)
@@ -998,8 +992,24 @@ void __weak boot_init_primary_late(unsigned long fdt __unused,
 		struct transfer_list_entry *tl_e = NULL;
 
 		tl_e = transfer_list_find(mapped_tl, TL_TAG_FDT);
-		if (tl_e)
+		if (tl_e) {
+			/*
+			 * Expand the data size of the DTB entry to the maximum
+			 * allocable mapped memory to reserve sufficient space
+			 * for inserting new nodes, avoid potentially corrupting
+			 * next entries.
+			 */
+			uint32_t dtb_max_sz = mapped_tl->max_size -
+					      mapped_tl->size + tl_e->data_size;
+
+			if (!transfer_list_set_data_size(mapped_tl, tl_e,
+							 dtb_max_sz)) {
+				EMSG("Failed to extend DTB size to %#"PRIx32,
+				     dtb_max_sz);
+				panic();
+			}
 			fdt_size = tl_e->data_size;
+		}
 	}
 
 	init_external_dt(boot_arg_fdt, fdt_size);
@@ -1013,6 +1023,24 @@ void __weak boot_init_primary_late(unsigned long fdt __unused,
 	update_external_dt();
 	configure_console_from_dt();
 
+	if (IS_ENABLED(CFG_NS_VIRTUALIZATION)) {
+		/*
+		 * Virtualization: We can't initialize threads right now because
+		 * threads belong to "tee" part and will be initialized
+		 * separately per each new virtual guest. So, we'll clear
+		 * "curr_thread" and call it done.
+		 */
+		thread_get_core_local()->curr_thread = -1;
+	} else {
+		thread_init_threads(CFG_NUM_THREADS);
+		thread_init_boot_thread();
+	}
+	thread_init_thread_core_local(CFG_TEE_CORE_NB_CORE);
+}
+
+void __weak boot_init_primary_runtime(void)
+{
+	thread_init_primary();
 	IMSG("OP-TEE version: %s", core_v_str);
 	if (IS_ENABLED(CFG_INSECURE)) {
 		IMSG("WARNING: This OP-TEE configuration might be insecure!");
@@ -1042,26 +1070,39 @@ void __weak boot_init_primary_late(unsigned long fdt __unused,
 
 	boot_primary_init_intc();
 	init_vfp_nsec();
-	if (!IS_ENABLED(CFG_NS_VIRTUALIZATION))
+	if (!IS_ENABLED(CFG_NS_VIRTUALIZATION)) {
+		/*
+		 * Unmask native interrupts during driver initcalls.
+		 *
+		 * NS-virtualization still uses the temporary stack also
+		 * used for exception handling so it must still have native
+		 * interrupts masked.
+		 */
+		thread_set_exceptions(thread_get_exceptions() &
+				      ~THREAD_EXCP_NATIVE_INTR);
 		init_tee_runtime();
-}
+	}
 
-/*
- * Note: this function is weak just to make it possible to exclude it from
- * the unpaged area.
- */
-void __weak boot_init_primary_final(void)
-{
 	if (!IS_ENABLED(CFG_WITH_PAGER))
 		boot_mem_release_tmp_alloc();
-
-	if (!IS_ENABLED(CFG_NS_VIRTUALIZATION))
-		call_driver_initcalls();
-	call_finalcalls();
-	IMSG("Primary CPU switching to normal world boot");
 }
 
-static void init_secondary_helper(unsigned long nsec_entry)
+void __weak boot_init_primary_final(void)
+{
+	if (!IS_ENABLED(CFG_NS_VIRTUALIZATION))
+		call_driver_initcalls();
+
+	call_finalcalls();
+
+	IMSG("Primary CPU switching to normal world boot");
+
+	/* Mask native interrupts before switching to the normal world */
+	if (!IS_ENABLED(CFG_NS_VIRTUALIZATION))
+		thread_set_exceptions(thread_get_exceptions() |
+				      THREAD_EXCP_NATIVE_INTR);
+}
+
+static void init_secondary_helper(void)
 {
 	IMSG("Secondary CPU %zu initializing", get_core_pos());
 
@@ -1076,7 +1117,6 @@ static void init_secondary_helper(unsigned long nsec_entry)
 
 	secondary_init_cntfrq();
 	thread_init_per_cpu();
-	init_sec_mon(nsec_entry);
 	boot_secondary_init_intc();
 	init_vfp_sec();
 	init_vfp_nsec();
@@ -1091,12 +1131,8 @@ static void init_secondary_helper(unsigned long nsec_entry)
 void __weak boot_init_primary_early(void)
 {
 	unsigned long pageable_part = 0;
-	unsigned long e = PADDR_INVALID;
 	struct transfer_list_entry *tl_e = NULL;
 
-	if (!IS_ENABLED(CFG_WITH_ARM_TRUSTED_FW))
-		e = boot_arg_nsec_entry;
-
 	if (IS_ENABLED(CFG_TRANSFER_LIST) && boot_arg_transfer_list) {
 		/* map and save the TL */
 		mapped_tl = transfer_list_map(boot_arg_transfer_list);
@@ -1104,24 +1140,6 @@ void __weak boot_init_primary_early(void)
 			panic("Failed to map transfer list");
 
 		transfer_list_dump(mapped_tl);
-		tl_e = transfer_list_find(mapped_tl, TL_TAG_FDT);
-		if (tl_e) {
-			/*
-			 * Expand the data size of the DTB entry to the maximum
-			 * allocable mapped memory to reserve sufficient space
-			 * for inserting new nodes, avoid potentially corrupting
-			 * next entries.
-			 */
-			uint32_t dtb_max_sz = mapped_tl->max_size -
-					      mapped_tl->size + tl_e->data_size;
-
-			if (!transfer_list_set_data_size(mapped_tl, tl_e,
-							 dtb_max_sz)) {
-				EMSG("Failed to extend DTB size to %#"PRIx32,
-				     dtb_max_sz);
-				panic();
-			}
-		}
 		tl_e = transfer_list_find(mapped_tl, TL_TAG_OPTEE_PAGABLE_PART);
 	}
 
@@ -1133,7 +1151,7 @@ void __weak boot_init_primary_early(void)
 			pageable_part = boot_arg_pageable_part;
 	}
 
-	init_primary(pageable_part, e);
+	init_primary(pageable_part);
 }
 
 static void boot_save_transfer_list(unsigned long zero_reg,
@@ -1164,13 +1182,13 @@ static void boot_save_transfer_list(unsigned long zero_reg,
 unsigned long boot_cpu_on_handler(unsigned long a0 __maybe_unused,
 				  unsigned long a1 __unused)
 {
-	init_secondary_helper(PADDR_INVALID);
+	init_secondary_helper();
 	return 0;
 }
 #else
-void boot_init_secondary(unsigned long nsec_entry)
+void boot_init_secondary(unsigned long nsec_entry __unused)
 {
-	init_secondary_helper(nsec_entry);
+	init_secondary_helper();
 }
 #endif
 
@@ -1277,7 +1295,8 @@ static void *get_fdt_from_boot_info(struct ffa_boot_info_header_1_1 *hdr)
 		EMSG("Bad boot info signature %#"PRIx32, hdr->signature);
 		panic();
 	}
-	if (hdr->version != FFA_BOOT_INFO_VERSION) {
+	if (hdr->version != FFA_BOOT_INFO_VERSION_1_1 &&
+	    hdr->version != FFA_BOOT_INFO_VERSION_1_2) {
 		EMSG("Bad boot info version %#"PRIx32, hdr->version);
 		panic();
 	}
@@ -1313,7 +1332,8 @@ static void *get_fdt_from_boot_info(struct ffa_boot_info_header_1_1 *hdr)
 	return fdt;
 }
 
-static void get_sec_mem_from_manifest(void *fdt, paddr_t *base, size_t *size)
+static void get_sec_mem_from_manifest(void *fdt, paddr_t *base,
+				      paddr_size_t *size)
 {
 	int ret = 0;
 	uint64_t num = 0;
@@ -1408,20 +1428,30 @@ void __weak boot_save_args(unsigned long a0, unsigned long a1,
 	}
 
 	if (IS_ENABLED(CFG_CORE_FFA)) {
+		size_t fdt_max_size = CFG_DTB_MAX_SIZE;
+		void *fdt = NULL;
+
 		if (IS_ENABLED(CFG_CORE_SEL2_SPMC) ||
 		    IS_ENABLED(CFG_CORE_EL3_SPMC))
-			init_manifest_dt(get_fdt_from_boot_info((void *)a0));
+			fdt = get_fdt_from_boot_info((void *)a0);
 		else
-			init_manifest_dt((void *)a0);
-		if (IS_ENABLED(CFG_CORE_SEL2_SPMC) &&
-		    IS_ENABLED(CFG_CORE_PHYS_RELOCATABLE)) {
+			fdt = (void *)a0;
+		if (IS_ENABLED(CFG_CORE_SEL2_SPMC)) {
+			paddr_size_t size = 0;
 			paddr_t base = 0;
-			size_t size = 0;
 
-			get_sec_mem_from_manifest(get_manifest_dt(),
-						  &base, &size);
-			core_mmu_set_secure_memory(base, size);
+			if (IS_ENABLED(CFG_CORE_PHYS_RELOCATABLE)) {
+				get_sec_mem_from_manifest(fdt, &base, &size);
+				core_mmu_set_secure_memory(base, size);
+			} else {
+				core_mmu_get_secure_memory(&base, &size);
+			}
+			assert((unsigned long)fdt >= base);
+			assert((unsigned long)fdt <= base + size);
+			assert((unsigned long)fdt < VCORE_START_VA);
+			fdt_max_size = VCORE_START_VA - (unsigned long)fdt;
 		}
+		init_manifest_dt(fdt, fdt_max_size);
 	} else {
 		if (IS_ENABLED(CFG_WITH_PAGER)) {
 #if defined(CFG_PAGEABLE_ADDR)
