@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: BSD-2-Clause
 /*
- * Copyright (c) 2020-2023, Linaro Limited.
+ * Copyright (c) 2020-2025, Linaro Limited.
  * Copyright (c) 2019-2024, Arm Limited. All rights reserved.
  */
 
@@ -8,6 +8,7 @@
 #include <ffa.h>
 #include <initcall.h>
 #include <io.h>
+#include <kernel/dt.h>
 #include <kernel/interrupt.h>
 #include <kernel/notif.h>
 #include <kernel/panic.h>
@@ -19,6 +20,7 @@
 #include <kernel/thread_private.h>
 #include <kernel/thread_spmc.h>
 #include <kernel/virtualization.h>
+#include <libfdt.h>
 #include <mm/core_mmu.h>
 #include <mm/mobj.h>
 #include <optee_ffa.h>
@@ -29,6 +31,8 @@
 #include <sys/queue.h>
 #include <tee/entry_std.h>
 #include <tee/uuid.h>
+#include <tee_api_types.h>
+#include <types_ext.h>
 #include <util.h>
 
 #if defined(CFG_CORE_SEL1_SPMC)
@@ -54,6 +58,11 @@ struct notif_vm_bitmap {
 	uint64_t bound;
 };
 
+STAILQ_HEAD(spmc_lsp_desc_head, spmc_lsp_desc);
+
+static struct spmc_lsp_desc_head lsp_head __nex_data =
+	STAILQ_HEAD_INITIALIZER(lsp_head);
+
 static unsigned int spmc_notif_lock __nex_data = SPINLOCK_UNLOCK;
 static bool spmc_notif_is_ready __nex_bss;
 static int notif_intid __nex_data __maybe_unused = -1;
@@ -64,32 +73,15 @@ static unsigned int notif_vm_bitmap_id __nex_bss;
 static struct notif_vm_bitmap default_notif_vm_bitmap;
 
 /* Initialized in spmc_init() below */
-uint16_t optee_endpoint_id __nex_bss;
-uint16_t spmc_id __nex_bss;
+static struct spmc_lsp_desc optee_core_lsp;
 #ifdef CFG_CORE_SEL1_SPMC
-uint16_t spmd_id __nex_bss;
-static const uint32_t my_part_props = FFA_PART_PROP_DIRECT_REQ_RECV |
-				      FFA_PART_PROP_DIRECT_REQ_SEND |
-#ifdef CFG_NS_VIRTUALIZATION
-				      FFA_PART_PROP_NOTIF_CREATED |
-				      FFA_PART_PROP_NOTIF_DESTROYED |
-#endif
-#ifdef ARM64
-				      FFA_PART_PROP_AARCH64_STATE |
-#endif
-				      FFA_PART_PROP_IS_PE_ID;
-
-static uint32_t my_uuid_words[] = {
-	/*
-	 * - if the SPMC is in S-EL2 this UUID describes OP-TEE as a S-EL1
-	 *   SP, or
-	 * - if the SPMC is in S-EL1 then this UUID is for OP-TEE as a
-	 *   logical partition, residing in the same exception level as the
-	 *   SPMC
-	 * UUID 486178e0-e7f8-11e3-bc5e-0002a5d5c51b
-	 */
-	0xe0786148, 0xe311f8e7, 0x02005ebc, 0x1bc5d5a5,
-};
+/*
+ * Representation of the internal SPMC when OP-TEE is the S-EL1 SPMC.
+ * Initialized in spmc_init() below.
+ */
+static struct spmc_lsp_desc optee_spmc_lsp;
+/* FF-A ID of the SPMD. This is only valid when OP-TEE is the S-EL1 SPMC. */
+static uint16_t spmd_id __nex_bss;
 
 /*
  * If struct ffa_rxtx::size is 0 RX/TX buffers are not mapped or initialized.
@@ -117,15 +109,37 @@ static SLIST_HEAD(mem_frag_state_head, mem_frag_state) frag_state_head =
 	SLIST_HEAD_INITIALIZER(&frag_state_head);
 
 #else
-static uint8_t __rx_buf[SMALL_PAGE_SIZE] __aligned(SMALL_PAGE_SIZE);
-static uint8_t __tx_buf[SMALL_PAGE_SIZE] __aligned(SMALL_PAGE_SIZE);
-static struct ffa_rxtx my_rxtx = {
+/* FF-A ID of the external SPMC */
+static uint16_t spmc_id __nex_bss;
+static uint8_t __rx_buf[SMALL_PAGE_SIZE] __aligned(SMALL_PAGE_SIZE) __nex_bss;
+static uint8_t __tx_buf[SMALL_PAGE_SIZE] __aligned(SMALL_PAGE_SIZE) __nex_bss;
+static struct ffa_rxtx my_rxtx __nex_data = {
 	.rx = __rx_buf,
 	.tx = __tx_buf,
 	.size = sizeof(__rx_buf),
 };
 #endif
 
+bool spmc_is_reserved_id(uint16_t id)
+{
+#ifdef CFG_CORE_SEL1_SPMC
+	return id == spmd_id;
+#else
+	return id == spmc_id;
+#endif
+}
+
+struct spmc_lsp_desc *spmc_find_lsp_by_sp_id(uint16_t sp_id)
+{
+	struct spmc_lsp_desc *desc = NULL;
+
+	STAILQ_FOREACH(desc, &lsp_head, link)
+		if (desc->sp_id == sp_id)
+			return desc;
+
+	return NULL;
+}
+
 static uint32_t swap_src_dst(uint32_t src_dst)
 {
 	return (src_dst >> 16) | (src_dst << 16);
@@ -136,18 +150,21 @@ static uint16_t get_sender_id(uint32_t src_dst)
 	return src_dst >> 16;
 }
 
-void spmc_set_args(struct thread_smc_args *args, uint32_t fid, uint32_t src_dst,
-		   uint32_t w2, uint32_t w3, uint32_t w4, uint32_t w5)
+void spmc_set_args(struct thread_smc_1_2_regs *args, uint32_t fid,
+		   uint32_t src_dst, uint32_t w2, uint32_t w3, uint32_t w4,
+		   uint32_t w5)
 {
-	*args = (struct thread_smc_args){ .a0 = fid,
-					  .a1 = src_dst,
-					  .a2 = w2,
-					  .a3 = w3,
-					  .a4 = w4,
-					  .a5 = w5, };
+	*args = (struct thread_smc_1_2_regs){
+		.a0 = fid,
+		.a1 = src_dst,
+		.a2 = w2,
+		.a3 = w3,
+		.a4 = w4,
+		.a5 = w5,
+	};
 }
 
-static void set_simple_ret_val(struct thread_smc_args *args, int ffa_ret)
+static void set_simple_ret_val(struct thread_smc_1_2_regs *args, int ffa_ret)
 {
 	if (ffa_ret)
 		spmc_set_args(args, FFA_ERROR, 0, ffa_ret, 0, 0, 0);
@@ -157,18 +174,35 @@ static void set_simple_ret_val(struct thread_smc_args *args, int ffa_ret)
 
 uint32_t spmc_exchange_version(uint32_t vers, struct ffa_rxtx *rxtx)
 {
+	uint32_t major_vers = FFA_GET_MAJOR_VERSION(vers);
+	uint32_t minor_vers = FFA_GET_MINOR_VERSION(vers);
+	uint32_t my_vers = FFA_VERSION_1_2;
+	uint32_t my_major_vers = 0;
+	uint32_t my_minor_vers = 0;
+
+	my_major_vers = FFA_GET_MAJOR_VERSION(my_vers);
+	my_minor_vers = FFA_GET_MINOR_VERSION(my_vers);
+
 	/*
 	 * No locking, if the caller does concurrent calls to this it's
 	 * only making a mess for itself. We must be able to renegotiate
 	 * the FF-A version in order to support differing versions between
 	 * the loader and the driver.
+	 *
+	 * Callers should use the version requested if we return a matching
+	 * major version and a matching or larger minor version. The caller
+	 * should downgrade to our minor version if our minor version is
+	 * smaller. Regardless, always return our version as recommended by
+	 * the specification.
 	 */
-	if (vers < FFA_VERSION_1_1)
-		rxtx->ffa_vers = FFA_VERSION_1_0;
-	else
-		rxtx->ffa_vers = FFA_VERSION_1_1;
+	if (major_vers == my_major_vers) {
+		if (minor_vers > my_minor_vers)
+			rxtx->ffa_vers = my_vers;
+		else
+			rxtx->ffa_vers = vers;
+	}
 
-	return rxtx->ffa_vers;
+	return my_vers;
 }
 
 static bool is_ffa_success(uint32_t fid)
@@ -219,7 +253,7 @@ static int __maybe_unused ffa_set_notification(uint16_t dst, uint16_t src,
 }
 
 #if defined(CFG_CORE_SEL1_SPMC)
-static void handle_features(struct thread_smc_args *args)
+static void handle_features(struct thread_smc_1_2_regs *args)
 {
 	uint32_t ret_fid = FFA_ERROR;
 	uint32_t ret_w2 = FFA_NOT_SUPPORTED;
@@ -312,9 +346,9 @@ static int map_buf(paddr_t pa, unsigned int sz, void **va_ret)
 	return 0;
 }
 
-void spmc_handle_spm_id_get(struct thread_smc_args *args)
+void spmc_handle_spm_id_get(struct thread_smc_1_2_regs *args)
 {
-	spmc_set_args(args, FFA_SUCCESS_32, FFA_PARAM_MBZ, spmc_id,
+	spmc_set_args(args, FFA_SUCCESS_32, FFA_PARAM_MBZ, optee_spmc_lsp.sp_id,
 		      FFA_PARAM_MBZ, FFA_PARAM_MBZ, FFA_PARAM_MBZ);
 }
 
@@ -327,7 +361,8 @@ static void unmap_buf(void *va, size_t sz)
 	tee_mm_free(mm);
 }
 
-void spmc_handle_rxtx_map(struct thread_smc_args *args, struct ffa_rxtx *rxtx)
+void spmc_handle_rxtx_map(struct thread_smc_1_2_regs *args,
+			  struct ffa_rxtx *rxtx)
 {
 	int rc = 0;
 	unsigned int sz = 0;
@@ -438,7 +473,8 @@ out:
 	set_simple_ret_val(args, rc);
 }
 
-void spmc_handle_rxtx_unmap(struct thread_smc_args *args, struct ffa_rxtx *rxtx)
+void spmc_handle_rxtx_unmap(struct thread_smc_1_2_regs *args,
+			    struct ffa_rxtx *rxtx)
 {
 	int rc = FFA_INVALID_PARAMETERS;
 
@@ -466,7 +502,8 @@ out:
 	set_simple_ret_val(args, rc);
 }
 
-void spmc_handle_rx_release(struct thread_smc_args *args, struct ffa_rxtx *rxtx)
+void spmc_handle_rx_release(struct thread_smc_1_2_regs *args,
+			    struct ffa_rxtx *rxtx)
 {
 	int rc = 0;
 
@@ -488,19 +525,6 @@ static bool is_nil_uuid(uint32_t w0, uint32_t w1, uint32_t w2, uint32_t w3)
 	return !w0 && !w1 && !w2 && !w3;
 }
 
-static bool is_my_uuid(uint32_t w0, uint32_t w1, uint32_t w2, uint32_t w3)
-{
-	/*
-	 * This depends on which UUID we have been assigned.
-	 * TODO add a generic mechanism to obtain our UUID.
-	 *
-	 * The test below is for the hard coded UUID
-	 * 486178e0-e7f8-11e3-bc5e-0002a5d5c51b
-	 */
-	return w0 == my_uuid_words[0] && w1 == my_uuid_words[1] &&
-	       w2 == my_uuid_words[2] && w3 == my_uuid_words[3];
-}
-
 TEE_Result spmc_fill_partition_entry(uint32_t ffa_vers, void *buf, size_t blen,
 				     size_t idx, uint16_t endpoint_id,
 				     uint16_t execution_context,
@@ -539,29 +563,43 @@ TEE_Result spmc_fill_partition_entry(uint32_t ffa_vers, void *buf, size_t blen,
 	return TEE_SUCCESS;
 }
 
-static int handle_partition_info_get_all(size_t *elem_count,
-					 struct ffa_rxtx *rxtx, bool count_only)
+static TEE_Result lsp_partition_info_get(uint32_t ffa_vers, void *buf,
+					 size_t buf_size, size_t *elem_count,
+					 const uint32_t uuid_words[4],
+					 bool count_only)
 {
-	if (!count_only) {
-		/* Add OP-TEE SP */
-		if (spmc_fill_partition_entry(rxtx->ffa_vers, rxtx->tx,
-					      rxtx->size, 0, optee_endpoint_id,
-					      CFG_TEE_CORE_NB_CORE,
-					      my_part_props, my_uuid_words))
-			return FFA_NO_MEMORY;
-	}
-	*elem_count = 1;
+	struct spmc_lsp_desc *desc = NULL;
+	TEE_Result res = TEE_SUCCESS;
+	size_t c = *elem_count;
 
-	if (IS_ENABLED(CFG_SECURE_PARTITION)) {
-		if (sp_partition_info_get(rxtx->ffa_vers, rxtx->tx, rxtx->size,
-					  NULL, elem_count, count_only))
-			return FFA_NO_MEMORY;
+	STAILQ_FOREACH(desc, &lsp_head, link) {
+		/*
+		 * LSPs (OP-TEE SPMC) without an assigned UUID are not
+		 * proper LSPs and shouldn't be reported here.
+		 */
+		if (is_nil_uuid(desc->uuid_words[0], desc->uuid_words[1],
+				desc->uuid_words[2], desc->uuid_words[3]))
+			continue;
+
+		if (uuid_words && memcmp(uuid_words, desc->uuid_words,
+					 sizeof(desc->uuid_words)))
+			continue;
+
+		if (!count_only && !res)
+			res = spmc_fill_partition_entry(ffa_vers, buf, buf_size,
+							c, desc->sp_id,
+							CFG_TEE_CORE_NB_CORE,
+							desc->properties,
+							desc->uuid_words);
+		c++;
 	}
 
-	return FFA_OK;
+	*elem_count = c;
+
+	return res;
 }
 
-void spmc_handle_partition_info_get(struct thread_smc_args *args,
+void spmc_handle_partition_info_get(struct thread_smc_1_2_regs *args,
 				    struct ffa_rxtx *rxtx)
 {
 	TEE_Result res = TEE_SUCCESS;
@@ -569,6 +607,9 @@ void spmc_handle_partition_info_get(struct thread_smc_args *args,
 	uint32_t fpi_size = 0;
 	uint32_t rc = 0;
 	bool count_only = args->a5 & FFA_PARTITION_INFO_GET_COUNT_FLAG;
+	uint32_t uuid_words[4] = { args->a1, args->a2, args->a3, args->a4, };
+	uint32_t *uuid = uuid_words;
+	size_t count = 0;
 
 	if (!count_only) {
 		cpu_spin_lock(&rxtx->spinlock);
@@ -579,66 +620,28 @@ void spmc_handle_partition_info_get(struct thread_smc_args *args,
 		}
 	}
 
-	if (is_nil_uuid(args->a1, args->a2, args->a3, args->a4)) {
-		size_t elem_count = 0;
-
-		ret_fid = handle_partition_info_get_all(&elem_count, rxtx,
-							count_only);
-
-		if (ret_fid) {
-			rc = ret_fid;
-			ret_fid = FFA_ERROR;
-		} else {
-			ret_fid = FFA_SUCCESS_32;
-			rc = elem_count;
-		}
+	if (is_nil_uuid(uuid[0], uuid[1], uuid[2], uuid[3]))
+		uuid = NULL;
 
+	if (lsp_partition_info_get(rxtx->ffa_vers, rxtx->tx, rxtx->size,
+				   &count, uuid, count_only)) {
+		ret_fid = FFA_ERROR;
+		rc = FFA_INVALID_PARAMETERS;
 		goto out;
 	}
-
-	if (is_my_uuid(args->a1, args->a2, args->a3, args->a4)) {
-		if (!count_only) {
-			res = spmc_fill_partition_entry(rxtx->ffa_vers,
-							rxtx->tx, rxtx->size, 0,
-							optee_endpoint_id,
-							CFG_TEE_CORE_NB_CORE,
-							my_part_props,
-							my_uuid_words);
-			if (res) {
-				ret_fid = FFA_ERROR;
-				rc = FFA_INVALID_PARAMETERS;
-				goto out;
-			}
-		}
-		rc = 1;
-	} else if (IS_ENABLED(CFG_SECURE_PARTITION)) {
-		uint32_t uuid_array[4] = { 0 };
-		TEE_UUID uuid = { };
-		size_t count = 0;
-
-		uuid_array[0] = args->a1;
-		uuid_array[1] = args->a2;
-		uuid_array[2] = args->a3;
-		uuid_array[3] = args->a4;
-		tee_uuid_from_octets(&uuid, (uint8_t *)uuid_array);
-
+	if (IS_ENABLED(CFG_SECURE_PARTITION)) {
 		res = sp_partition_info_get(rxtx->ffa_vers, rxtx->tx,
-					    rxtx->size, &uuid, &count,
+					    rxtx->size, uuid, &count,
 					    count_only);
 		if (res != TEE_SUCCESS) {
 			ret_fid = FFA_ERROR;
 			rc = FFA_INVALID_PARAMETERS;
 			goto out;
 		}
-		rc = count;
-	} else {
-		ret_fid = FFA_ERROR;
-		rc = FFA_INVALID_PARAMETERS;
-		goto out;
 	}
 
+	rc = count;
 	ret_fid = FFA_SUCCESS_32;
-
 out:
 	if (ret_fid == FFA_SUCCESS_32 && !count_only &&
 	    rxtx->ffa_vers >= FFA_VERSION_1_1)
@@ -652,25 +655,34 @@ out:
 	}
 }
 
-static void spmc_handle_run(struct thread_smc_args *args)
+static void spmc_handle_run(struct thread_smc_1_2_regs *args)
 {
 	uint16_t endpoint = FFA_TARGET_INFO_GET_SP_ID(args->a1);
 	uint16_t thread_id = FFA_TARGET_INFO_GET_VCPU_ID(args->a1);
-	uint32_t rc = FFA_OK;
+	uint32_t rc = FFA_INVALID_PARAMETERS;
 
-	if (endpoint != optee_endpoint_id) {
-		/*
-		 * The endpoint should be an SP, try to resume the SP from
-		 * preempted into busy state.
-		 */
-		rc = spmc_sp_resume_from_preempted(endpoint);
-		if (rc)
-			goto out;
-	}
+	/*
+	 * OP-TEE core threads are only preemted using controlled exit so
+	 * FFA_RUN mustn't be used to resume such threads.
+	 *
+	 * The OP-TEE SPMC is not preemted at all, it's an error to try to
+	 * resume that ID.
+	 */
+	if (spmc_find_lsp_by_sp_id(endpoint))
+		goto out;
 
+	/*
+	 * The endpoint should be a S-EL0 SP, try to resume the SP from
+	 * preempted into busy state.
+	 */
+	rc = spmc_sp_resume_from_preempted(endpoint);
+	if (rc)
+		goto out;
 	thread_resume_from_rpc(thread_id, 0, 0, 0, 0);
-
-	/* thread_resume_from_rpc return only of the thread_id is invalid */
+	/*
+	 * thread_resume_from_rpc() only returns if the thread_id
+	 * is invalid.
+	 */
 	rc = FFA_INVALID_PARAMETERS;
 
 out:
@@ -732,10 +744,20 @@ out:
 	return res;
 }
 
-static void handle_yielding_call(struct thread_smc_args *args,
-				 uint32_t direct_resp_fid)
+static uint32_t get_direct_resp_fid(uint32_t fid)
 {
-	TEE_Result res = 0;
+	assert(fid == FFA_MSG_SEND_DIRECT_REQ_64 ||
+	       fid == FFA_MSG_SEND_DIRECT_REQ_32);
+
+	if (OPTEE_SMC_IS_64(fid))
+		return FFA_MSG_SEND_DIRECT_RESP_64;
+	return FFA_MSG_SEND_DIRECT_RESP_32;
+}
+
+static void handle_yielding_call(struct thread_smc_1_2_regs *args)
+{
+	uint32_t direct_resp_fid = get_direct_resp_fid(args->a0);
+	TEE_Result res = TEE_SUCCESS;
 
 	thread_check_canaries();
 
@@ -777,9 +799,9 @@ static uint32_t handle_unregister_shm(uint32_t a4, uint32_t a5)
 	}
 }
 
-static void handle_blocking_call(struct thread_smc_args *args,
-				 uint32_t direct_resp_fid)
+static void handle_blocking_call(struct thread_smc_1_2_regs *args)
 {
+	uint32_t direct_resp_fid = get_direct_resp_fid(args->a0);
 	uint32_t sec_caps = 0;
 
 	switch (args->a3) {
@@ -823,10 +845,9 @@ static void handle_blocking_call(struct thread_smc_args *args,
 	}
 }
 
-static void handle_framework_direct_request(struct thread_smc_args *args,
-					    struct ffa_rxtx *rxtx,
-					    uint32_t direct_resp_fid)
+static void handle_framework_direct_request(struct thread_smc_1_2_regs *args)
 {
+	uint32_t direct_resp_fid = get_direct_resp_fid(args->a0);
 	uint32_t w0 = FFA_ERROR;
 	uint32_t w1 = FFA_PARAM_MBZ;
 	uint32_t w2 = FFA_NOT_SUPPORTED;
@@ -867,7 +888,7 @@ static void handle_framework_direct_request(struct thread_smc_args *args,
 		w0 = direct_resp_fid;
 		w1 = swap_src_dst(args->a1);
 		w2 = FFA_MSG_FLAG_FRAMEWORK | FFA_MSG_VERSION_RESP;
-		w3 = spmc_exchange_version(args->a3, rxtx);
+		w3 = spmc_exchange_version(args->a3, &my_rxtx);
 		break;
 	default:
 		break;
@@ -875,39 +896,25 @@ static void handle_framework_direct_request(struct thread_smc_args *args,
 	spmc_set_args(args, w0, w1, w2, w3, FFA_PARAM_MBZ, FFA_PARAM_MBZ);
 }
 
-static void handle_direct_request(struct thread_smc_args *args,
-				  struct ffa_rxtx *rxtx)
+static void optee_lsp_handle_direct_request(struct thread_smc_1_2_regs *args)
 {
-	uint32_t direct_resp_fid = 0;
-
-	if (IS_ENABLED(CFG_SECURE_PARTITION) &&
-	    FFA_DST(args->a1) != spmc_id &&
-	    FFA_DST(args->a1) != optee_endpoint_id) {
-		spmc_sp_start_thread(args);
-		return;
-	}
-
-	if (OPTEE_SMC_IS_64(args->a0))
-		direct_resp_fid = FFA_MSG_SEND_DIRECT_RESP_64;
-	else
-		direct_resp_fid = FFA_MSG_SEND_DIRECT_RESP_32;
-
 	if (args->a2 & FFA_MSG_FLAG_FRAMEWORK) {
-		handle_framework_direct_request(args, rxtx, direct_resp_fid);
+		handle_framework_direct_request(args);
 		return;
 	}
 
 	if (IS_ENABLED(CFG_NS_VIRTUALIZATION) &&
 	    virt_set_guest(get_sender_id(args->a1))) {
-		spmc_set_args(args, direct_resp_fid, swap_src_dst(args->a1), 0,
+		spmc_set_args(args, get_direct_resp_fid(args->a0),
+			      swap_src_dst(args->a1), 0,
 			      TEE_ERROR_ITEM_NOT_FOUND, 0, 0);
 		return;
 	}
 
 	if (args->a3 & BIT32(OPTEE_FFA_YIELDING_CALL_BIT))
-		handle_yielding_call(args, direct_resp_fid);
+		handle_yielding_call(args);
 	else
-		handle_blocking_call(args, direct_resp_fid);
+		handle_blocking_call(args);
 
 	/*
 	 * Note that handle_yielding_call() typically only returns if a
@@ -918,6 +925,31 @@ static void handle_direct_request(struct thread_smc_args *args,
 		virt_unset_guest();
 }
 
+static void __maybe_unused
+optee_spmc_lsp_handle_direct_request(struct thread_smc_1_2_regs *args)
+{
+	if (args->a2 & FFA_MSG_FLAG_FRAMEWORK)
+		handle_framework_direct_request(args);
+	else
+		set_simple_ret_val(args, FFA_INVALID_PARAMETERS);
+}
+
+static void handle_direct_request(struct thread_smc_1_2_regs *args)
+{
+	struct spmc_lsp_desc *lsp = spmc_find_lsp_by_sp_id(FFA_DST(args->a1));
+
+	if (lsp) {
+		lsp->direct_req(args);
+	} else {
+		spmc_sp_start_thread(args);
+		/*
+		 * spmc_sp_start_thread() returns here if the SP ID is
+		 * invalid.
+		 */
+		set_simple_ret_val(args, FFA_INVALID_PARAMETERS);
+	}
+}
+
 int spmc_read_mem_transaction(uint32_t ffa_vers, void *buf, size_t blen,
 			      struct ffa_mem_transaction_x *trans)
 {
@@ -995,7 +1027,7 @@ static int get_acc_perms(vaddr_t mem_acc_base, unsigned int mem_access_size,
 	for (n = 0; n < mem_access_count; n++) {
 		mem_acc = (void *)(mem_acc_base + mem_access_size * n);
 		descr = &mem_acc->access_perm;
-		if (READ_ONCE(descr->endpoint_id) == optee_endpoint_id) {
+		if (READ_ONCE(descr->endpoint_id) == optee_core_lsp.sp_id) {
 			*acc_perms = READ_ONCE(descr->perm);
 			*region_offs = READ_ONCE(mem_acc[n].region_offs);
 			return 0;
@@ -1125,7 +1157,7 @@ static bool is_sp_share(struct ffa_mem_transaction_x *mem_trans,
 	 * OP-TEE. We do read it later on again, but there are some additional
 	 * checks there to make sure that the data is correct.
 	 */
-	return READ_ONCE(perm->endpoint_id) != optee_endpoint_id;
+	return READ_ONCE(perm->endpoint_id) != optee_core_lsp.sp_id;
 }
 
 static int add_mem_share(struct ffa_mem_transaction_x *mem_trans,
@@ -1302,7 +1334,7 @@ out:
 	return rc;
 }
 
-static void handle_mem_share(struct thread_smc_args *args,
+static void handle_mem_share(struct thread_smc_1_2_regs *args,
 			     struct ffa_rxtx *rxtx)
 {
 	uint32_t tot_len = args->a1;
@@ -1366,7 +1398,7 @@ static struct mem_frag_state *get_frag_state(uint64_t global_handle)
 	return NULL;
 }
 
-static void handle_mem_frag_tx(struct thread_smc_args *args,
+static void handle_mem_frag_tx(struct thread_smc_1_2_regs *args,
 			       struct ffa_rxtx *rxtx)
 {
 	uint64_t global_handle = reg_pair_to_64(args->a2, args->a1);
@@ -1448,7 +1480,7 @@ out_set_rc:
 	spmc_set_args(args, ret_fid, ret_w1, ret_w2, ret_w3, 0, 0);
 }
 
-static void handle_mem_reclaim(struct thread_smc_args *args)
+static void handle_mem_reclaim(struct thread_smc_1_2_regs *args)
 {
 	int rc = FFA_INVALID_PARAMETERS;
 	uint64_t cookie = 0;
@@ -1497,7 +1529,7 @@ out:
 	set_simple_ret_val(args, rc);
 }
 
-static void handle_notification_bitmap_create(struct thread_smc_args *args)
+static void handle_notification_bitmap_create(struct thread_smc_1_2_regs *args)
 {
 	uint32_t ret_val = FFA_INVALID_PARAMETERS;
 	uint32_t ret_fid = FFA_ERROR;
@@ -1536,7 +1568,7 @@ out_virt_put:
 	spmc_set_args(args, ret_fid, 0, ret_val, 0, 0, 0);
 }
 
-static void handle_notification_bitmap_destroy(struct thread_smc_args *args)
+static void handle_notification_bitmap_destroy(struct thread_smc_1_2_regs *args)
 {
 	uint32_t ret_val = FFA_INVALID_PARAMETERS;
 	uint32_t ret_fid = FFA_ERROR;
@@ -1574,7 +1606,7 @@ out_virt_put:
 	spmc_set_args(args, ret_fid, 0, ret_val, 0, 0, 0);
 }
 
-static void handle_notification_bind(struct thread_smc_args *args)
+static void handle_notification_bind(struct thread_smc_1_2_regs *args)
 {
 	uint32_t ret_val = FFA_INVALID_PARAMETERS;
 	struct guest_partition *prtn = NULL;
@@ -1620,7 +1652,7 @@ out:
 	spmc_set_args(args, ret_fid, 0, ret_val, 0, 0, 0);
 }
 
-static void handle_notification_unbind(struct thread_smc_args *args)
+static void handle_notification_unbind(struct thread_smc_1_2_regs *args)
 {
 	uint32_t ret_val = FFA_INVALID_PARAMETERS;
 	struct guest_partition *prtn = NULL;
@@ -1661,7 +1693,7 @@ out:
 	spmc_set_args(args, ret_fid, 0, ret_val, 0, 0, 0);
 }
 
-static void handle_notification_get(struct thread_smc_args *args)
+static void handle_notification_get(struct thread_smc_1_2_regs *args)
 {
 	uint32_t w2 = FFA_INVALID_PARAMETERS;
 	struct guest_partition *prtn = NULL;
@@ -1699,7 +1731,7 @@ out:
 }
 
 struct notif_info_get_state {
-	struct thread_smc_args *args;
+	struct thread_smc_1_2_regs *args;
 	unsigned int ids_per_reg;
 	unsigned int ids_count;
 	unsigned int id_pos;
@@ -1708,78 +1740,17 @@ struct notif_info_get_state {
 	unsigned int list_count;
 };
 
-static unsigned long get_smc_arg(struct thread_smc_args *args, unsigned int idx)
-{
-	switch (idx) {
-	case 0:
-		return args->a0;
-	case 1:
-		return args->a1;
-	case 2:
-		return args->a2;
-	case 3:
-		return args->a3;
-	case 4:
-		return args->a4;
-	case 5:
-		return args->a5;
-	case 6:
-		return args->a6;
-	case 7:
-		return args->a7;
-	default:
-		assert(0);
-		return 0;
-	}
-}
-
-static void set_smc_arg(struct thread_smc_args *args, unsigned int idx,
-			unsigned long val)
-{
-	switch (idx) {
-	case 0:
-		args->a0 = val;
-		break;
-	case 1:
-		args->a1 = val;
-		break;
-	case 2:
-		args->a2 = val;
-		break;
-	case 3:
-		args->a3 = val;
-		break;
-	case 4:
-		args->a4 = val;
-		break;
-	case 5:
-		args->a5 = val;
-		break;
-	case 6:
-		args->a6 = val;
-		break;
-	case 7:
-		args->a7 = val;
-		break;
-	default:
-		assert(0);
-	}
-}
-
 static bool add_id_in_regs(struct notif_info_get_state *state,
 			   uint16_t id)
 {
 	unsigned int reg_idx = state->id_pos / state->ids_per_reg + 3;
 	unsigned int reg_shift = (state->id_pos % state->ids_per_reg) * 16;
-	unsigned long v;
 
 	if (reg_idx > 7)
 		return false;
 
-	v = get_smc_arg(state->args, reg_idx);
-	v &= ~(0xffffUL << reg_shift);
-	v |= (unsigned long)id << reg_shift;
-	set_smc_arg(state->args, reg_idx, v);
+	state->args->a[reg_idx] &= ~SHIFT_U64(0xffff, reg_shift);
+	state->args->a[reg_idx] |= (unsigned long)id << reg_shift;
 
 	state->id_pos++;
 	state->count++;
@@ -1813,7 +1784,7 @@ static bool add_nvb_to_state(struct notif_info_get_state *state,
 	return add_id_in_regs(state, guest_id) && add_id_count(state);
 }
 
-static void handle_notification_info_get(struct thread_smc_args *args)
+static void handle_notification_info_get(struct thread_smc_1_2_regs *args)
 {
 	struct notif_info_get_state state = { .args = args };
 	uint32_t ffa_res = FFA_INVALID_PARAMETERS;
@@ -1918,7 +1889,8 @@ void notif_send_async(uint32_t value, uint16_t guest_id)
 	if (nvb) {
 		assert(value == NOTIF_VALUE_DO_BOTTOM_HALF &&
 		       spmc_notif_is_ready && nvb->do_bottom_half_value >= 0);
-		res = ffa_set_notification(guest_id, optee_endpoint_id, flags,
+		res = ffa_set_notification(guest_id, optee_core_lsp.sp_id,
+					   flags,
 					   BIT64(nvb->do_bottom_half_value));
 		if (res) {
 			EMSG("notification set failed with error %d", res);
@@ -1931,8 +1903,8 @@ void notif_send_async(uint32_t value, uint16_t guest_id)
 #endif
 
 /* Only called from assembly */
-void thread_spmc_msg_recv(struct thread_smc_args *args);
-void thread_spmc_msg_recv(struct thread_smc_args *args)
+void thread_spmc_msg_recv(struct thread_smc_1_2_regs *args);
+void thread_spmc_msg_recv(struct thread_smc_1_2_regs *args)
 {
 	assert((thread_get_exceptions() & THREAD_EXCP_ALL) == THREAD_EXCP_ALL);
 	switch (args->a0) {
@@ -1973,7 +1945,7 @@ void thread_spmc_msg_recv(struct thread_smc_args *args)
 	case FFA_MSG_SEND_DIRECT_REQ_64:
 #endif
 	case FFA_MSG_SEND_DIRECT_REQ_32:
-		handle_direct_request(args, &my_rxtx);
+		handle_direct_request(args);
 		break;
 #if defined(CFG_CORE_SEL1_SPMC)
 #ifdef ARM64
@@ -2390,7 +2362,117 @@ static uint16_t ffa_spm_id_get(void)
 	return args.a2;
 }
 
+static TEE_Result check_desc(struct spmc_lsp_desc *d)
+{
+	uint32_t accept_props = FFA_PART_PROP_DIRECT_REQ_RECV |
+				FFA_PART_PROP_DIRECT_REQ_SEND |
+				FFA_PART_PROP_NOTIF_CREATED |
+				FFA_PART_PROP_NOTIF_DESTROYED |
+				FFA_PART_PROP_AARCH64_STATE;
+	uint32_t id = d->sp_id;
+
+	if (id && (spmc_is_reserved_id(id) || spmc_find_lsp_by_sp_id(id) ||
+		   id < FFA_SWD_ID_MIN || id > FFA_SWD_ID_MAX)) {
+		EMSG("Conflicting SP id for SP \"%s\" id %#"PRIx32,
+		     d->name, id);
+		if (!IS_ENABLED(CFG_SP_SKIP_FAILED))
+			panic();
+		return TEE_ERROR_BAD_FORMAT;
+	}
+
+	if (d->properties & ~accept_props) {
+		EMSG("Unexpected properties in %#"PRIx32" for LSP \"%s\" %#"PRIx16,
+		     d->properties, d->name, d->sp_id);
+		if (!IS_ENABLED(CFG_SP_SKIP_FAILED))
+			panic();
+		d->properties &= accept_props;
+	}
+
+	if (!d->direct_req) {
+		EMSG("Missing direct request callback for LSP \"%s\" %#"PRIx16,
+		     d->name, d->sp_id);
+		if (!IS_ENABLED(CFG_SP_SKIP_FAILED))
+			panic();
+		return TEE_ERROR_BAD_FORMAT;
+	}
+
+	if (!d->uuid_words[0] && !d->uuid_words[1] &&
+	    !d->uuid_words[2] && !d->uuid_words[3]) {
+		EMSG("Found NULL UUID for LSP \"%s\" %#"PRIx16,
+		     d->name, d->sp_id);
+		if (!IS_ENABLED(CFG_SP_SKIP_FAILED))
+			panic();
+		return TEE_ERROR_BAD_FORMAT;
+	}
+
+	return TEE_SUCCESS;
+}
+
+static uint16_t find_unused_sp_id(void)
+{
+	uint32_t id = FFA_SWD_ID_MIN;
+
+	while (spmc_is_reserved_id(id) || spmc_find_lsp_by_sp_id(id)) {
+		id++;
+		assert(id <= FFA_SWD_ID_MAX);
+	}
+
+	return id;
+}
+
+TEE_Result spmc_register_lsp(struct spmc_lsp_desc *desc)
+{
+	TEE_Result res = TEE_SUCCESS;
+
+	res = check_desc(desc);
+	if (res)
+		return res;
+
+	if (STAILQ_EMPTY(&lsp_head)) {
+		DMSG("Cannot add Logical SP \"%s\": LSP framework not initialized yet",
+		     desc->name);
+		return TEE_ERROR_ITEM_NOT_FOUND;
+	}
+
+	if (!desc->sp_id)
+		desc->sp_id = find_unused_sp_id();
+
+	DMSG("Adding Logical SP \"%s\" with id %#"PRIx16,
+	     desc->name, desc->sp_id);
+
+	STAILQ_INSERT_TAIL(&lsp_head, desc, link);
+
+	return TEE_SUCCESS;
+}
+
+static struct spmc_lsp_desc optee_core_lsp __nex_data = {
+	.name = "OP-TEE",
+	.direct_req = optee_lsp_handle_direct_request,
+	.properties = FFA_PART_PROP_DIRECT_REQ_RECV |
+		      FFA_PART_PROP_DIRECT_REQ_SEND |
+#ifdef CFG_NS_VIRTUALIZATION
+		      FFA_PART_PROP_NOTIF_CREATED |
+		      FFA_PART_PROP_NOTIF_DESTROYED |
+#endif
+		      FFA_PART_PROP_AARCH64_STATE |
+		      FFA_PART_PROP_IS_PE_ID,
+	/*
+	 * - if the SPMC is in S-EL2 this UUID describes OP-TEE as a S-EL1
+	 *   SP, or
+	 * - if the SPMC is in S-EL1 then this UUID is for OP-TEE as a
+	 *   logical partition, residing in the same exception level as the
+	 *   SPMC
+	 * UUID 486178e0-e7f8-11e3-bc5e-0002a5d5c51b
+	 */
+	.uuid_words = { 0xe0786148, 0xe311f8e7, 0x02005ebc, 0x1bc5d5a5, },
+};
+
 #if defined(CFG_CORE_SEL1_SPMC)
+static struct spmc_lsp_desc optee_spmc_lsp __nex_data = {
+	.name = "OP-TEE SPMC",
+	.direct_req = optee_spmc_lsp_handle_direct_request,
+};
+
 static TEE_Result spmc_init(void)
 {
 	if (IS_ENABLED(CFG_NS_VIRTUALIZATION) &&
@@ -2400,14 +2482,13 @@ static TEE_Result spmc_init(void)
 	spmd_id = ffa_spm_id_get();
 	DMSG("SPMD ID %#"PRIx16, spmd_id);
 
-	spmc_id = ffa_id_get();
-	DMSG("SPMC ID %#"PRIx16, spmc_id);
+	optee_spmc_lsp.sp_id = ffa_id_get();
+	DMSG("SPMC ID %#"PRIx16, optee_spmc_lsp.sp_id);
+	STAILQ_INSERT_HEAD(&lsp_head, &optee_spmc_lsp, link);
 
-	optee_endpoint_id = FFA_SWD_ID_MIN;
-	while (optee_endpoint_id == spmd_id || optee_endpoint_id == spmc_id)
-		optee_endpoint_id++;
-
-	DMSG("OP-TEE endpoint ID %#"PRIx16, optee_endpoint_id);
+	optee_core_lsp.sp_id = find_unused_sp_id();
+	DMSG("OP-TEE endpoint ID %#"PRIx16, optee_core_lsp.sp_id);
+	STAILQ_INSERT_HEAD(&lsp_head, &optee_core_lsp, link);
 
 	/*
 	 * If SPMD think we are version 1.0 it will report version 1.0 to
@@ -2507,7 +2588,7 @@ static void *spmc_retrieve_req(uint64_t cookie,
 	acc_descr_array->region_offs = 0;
 	acc_descr_array->reserved = 0;
 	perm_descr = &acc_descr_array->access_perm;
-	perm_descr->endpoint_id = optee_endpoint_id;
+	perm_descr->endpoint_id = optee_core_lsp.sp_id;
 	perm_descr->perm = FFA_MEM_ACC_RW;
 	perm_descr->flags = 0;
 
@@ -2545,7 +2626,7 @@ void thread_spmc_relinquish(uint64_t cookie)
 	relinquish_desc->handle = cookie;
 	relinquish_desc->flags = 0;
 	relinquish_desc->endpoint_count = 1;
-	relinquish_desc->endpoint_id_array[0] = optee_endpoint_id;
+	relinquish_desc->endpoint_id_array[0] = optee_core_lsp.sp_id;
 	thread_smccc(&args);
 	if (!is_ffa_success(args.a0))
 		EMSG("Failed to relinquish cookie %#"PRIx64, cookie);
@@ -2621,28 +2702,57 @@ out:
 	return ret;
 }
 
+static uint32_t get_ffa_version_from_manifest(void *fdt)
+{
+	int ret = 0;
+	uint32_t vers = 0;
+
+	ret = fdt_node_check_compatible(fdt, 0, "arm,ffa-manifest-1.0");
+	if (ret < 0) {
+		EMSG("Invalid FF-A manifest at %p: error %d", fdt, ret);
+		panic();
+	}
+
+	ret = fdt_read_uint32(fdt, 0, "ffa-version", &vers);
+	if (ret < 0) {
+		EMSG("Can't read \"ffa-version\" from FF-A manifest at %p: error %d",
+		     fdt, ret);
+		panic();
+	}
+
+	return vers;
+}
+
 static TEE_Result spmc_init(void)
 {
-	unsigned int major = 0;
-	unsigned int minor __maybe_unused = 0;
 	uint32_t my_vers = 0;
 	uint32_t vers = 0;
 
-	my_vers = MAKE_FFA_VERSION(FFA_VERSION_MAJOR, FFA_VERSION_MINOR);
+	if (IS_ENABLED(CFG_NS_VIRTUALIZATION) &&
+	    virt_add_guest_spec_data(&notif_vm_bitmap_id,
+				     sizeof(struct notif_vm_bitmap), NULL))
+		panic("virt_add_guest_spec_data");
+
+	my_vers = get_ffa_version_from_manifest(get_manifest_dt());
+	if (my_vers < FFA_VERSION_1_0 || my_vers > FFA_VERSION_1_2) {
+		EMSG("Unsupported version %"PRIu32".%"PRIu32" from manifest",
+		     FFA_GET_MAJOR_VERSION(my_vers),
+		     FFA_GET_MINOR_VERSION(my_vers));
+		panic();
+	}
 	vers = get_ffa_version(my_vers);
-	major = (vers >> FFA_VERSION_MAJOR_SHIFT) & FFA_VERSION_MAJOR_MASK;
-	minor = (vers >> FFA_VERSION_MINOR_SHIFT) & FFA_VERSION_MINOR_MASK;
-	DMSG("SPMC reported version %u.%u", major, minor);
-	if (major != FFA_VERSION_MAJOR) {
-		EMSG("Incompatible major version %u, expected %u",
-		     major, FFA_VERSION_MAJOR);
+	DMSG("SPMC reported version %"PRIu32".%"PRIu32,
+	     FFA_GET_MAJOR_VERSION(vers), FFA_GET_MINOR_VERSION(vers));
+	if (FFA_GET_MAJOR_VERSION(vers) != FFA_GET_MAJOR_VERSION(my_vers)) {
+		EMSG("Incompatible major version %"PRIu32", expected %"PRIu32"",
+		     FFA_GET_MAJOR_VERSION(vers),
+		     FFA_GET_MAJOR_VERSION(my_vers));
 		panic();
 	}
 	if (vers < my_vers)
 		my_vers = vers;
-	DMSG("Using version %u.%u",
-	     (my_vers >> FFA_VERSION_MAJOR_SHIFT) & FFA_VERSION_MAJOR_MASK,
-	     (my_vers >> FFA_VERSION_MINOR_SHIFT) & FFA_VERSION_MINOR_MASK);
+	DMSG("Using version %"PRIu32".%"PRIu32"",
+	     FFA_GET_MAJOR_VERSION(my_vers), FFA_GET_MINOR_VERSION(my_vers));
 	my_rxtx.ffa_vers = my_vers;
 
 	spmc_rxtx_map(&my_rxtx);
@@ -2650,8 +2760,9 @@ static TEE_Result spmc_init(void)
 	spmc_id = ffa_spm_id_get();
 	DMSG("SPMC ID %#"PRIx16, spmc_id);
 
-	optee_endpoint_id = ffa_id_get();
-	DMSG("OP-TEE endpoint ID %#"PRIx16, optee_endpoint_id);
+	optee_core_lsp.sp_id = ffa_id_get();
+	DMSG("OP-TEE endpoint ID %#"PRIx16, optee_core_lsp.sp_id);
+	STAILQ_INSERT_HEAD(&lsp_head, &optee_core_lsp, link);
 
 	if (!ffa_features(FFA_NOTIFICATION_SET)) {
 		spmc_notif_is_ready = true;
