@@ -49,6 +49,7 @@ void plat_console_init(void)
 	register_serial_console(&console_data.chip);
 }
 
+#ifndef PLATFORM_FLAVOR_am62lx
 static TEE_Result init_ti_sci(void)
 {
 	TEE_Result ret = TEE_SUCCESS;
@@ -111,3 +112,4 @@ TEE_Result tee_otp_get_hw_unique_key(struct tee_hw_unique_key *hwkey)
 
 	return TEE_SUCCESS;
 }
+#endif /* PLATFORM_FLAVOR_am62lx */
