@@ -49,6 +49,7 @@
 #define BSEC_BASE			0x5c005000
 #define ETZPC_BASE			0x5c007000
 #define CRYP1_BASE			0x54001000
+#define DBGMCU_BASE			0x50081000
 #define DDR_BASE			0xc0000000ul
 #define GIC_BASE			0xa0021000ul
 #define GPIOA_BASE			0x50002000
@@ -199,6 +200,12 @@
 #define USART3_BASE			UART3_BASE
 #define USART6_BASE			UART6_BASE
 
+/* DBGMCU resources */
+#define DBGMCU_IDC			U(0x0)
+#define DBGMCU_IDC_DEV_ID_MASK		GENMASK_32(11, 0)
+#define DBGMCU_IDC_REV_ID_MASK		GENMASK_32(31, 16)
+#define DBGMCU_IDC_REV_ID_SHIFT		U(16)
+
 /* SYSRAM layout */
 #ifdef CFG_STM32MP13
 #define SYSRAM_SIZE			0x20000
@@ -249,4 +256,39 @@
 #define SRAM3_SIZE			0x2000
 #endif
 
+/* SoC part numbers and revisions */
+#define STM32MP135C_PART_NB		U(0x05010000)
+#define STM32MP135A_PART_NB		U(0x05010001)
+#define STM32MP133C_PART_NB		U(0x050100C0)
+#define STM32MP133A_PART_NB		U(0x050100C1)
+#define STM32MP131C_PART_NB		U(0x050106C8)
+#define STM32MP131A_PART_NB		U(0x050106C9)
+#define STM32MP135F_PART_NB		U(0x05010800)
+#define STM32MP135D_PART_NB		U(0x05010801)
+#define STM32MP133F_PART_NB		U(0x050108C0)
+#define STM32MP133D_PART_NB		U(0x050108C1)
+#define STM32MP131F_PART_NB		U(0x05010EC8)
+#define STM32MP131D_PART_NB		U(0x05010EC9)
+
+#define STM32MP157C_PART_NB		U(0x05000000)
+#define STM32MP157A_PART_NB		U(0x05000001)
+#define STM32MP153C_PART_NB		U(0x05000024)
+#define STM32MP153A_PART_NB		U(0x05000025)
+#define STM32MP151C_PART_NB		U(0x0500002E)
+#define STM32MP151A_PART_NB		U(0x0500002F)
+#define STM32MP157F_PART_NB		U(0x05000080)
+#define STM32MP157D_PART_NB		U(0x05000081)
+#define STM32MP153F_PART_NB		U(0x050000A4)
+#define STM32MP153D_PART_NB		U(0x050000A5)
+#define STM32MP151F_PART_NB		U(0x050000AE)
+#define STM32MP151D_PART_NB		U(0x050000AF)
+
+#ifdef CFG_STM32MP13
+#define STM32MP1_CHIP_ID		U(0x501)
+#endif
+
+#ifdef CFG_STM32MP15
+#define STM32MP1_CHIP_ID		U(0x500)
+#endif
+
 #endif /*PLATFORM_CONFIG_H*/
