@@ -6,4 +6,7 @@
 #ifndef __KERNEL_MISC_ARCH_H
 #define __KERNEL_MISC_ARCH_H
 
+/* Bootable hart ID table */
+extern uint32_t hartids[CFG_TEE_CORE_NB_CORE];
+
 #endif /*__KERNEL_MISC_ARCH_H*/
