@@ -7,6 +7,7 @@
 #define __DRIVERS_STM32_GPIO_H
 
 #include <assert.h>
+#include <drivers/gpio.h>
 #include <drivers/pinctrl.h>
 #include <stdbool.h>
 #include <stdint.h>
@@ -26,6 +27,12 @@ struct stm32_pinctrl;
 void stm32_gpio_pinctrl_bank_pin(struct pinctrl_state *pinctrl,
 				 unsigned int *bank, unsigned int *pin,
 				 unsigned int *count);
+
+/*
+ * stm32_gpio_chip_bank_id() - Get the GPIO bank ID associated to a chip
+ * @chip: GPIO chip associated to the bank
+ */
+unsigned int stm32_gpio_chip_bank_id(struct gpio_chip *chip);
 #else
 static inline void stm32_gpio_pinctrl_bank_pin(struct pinctrl_state *p __unused,
 					       unsigned int *bank __unused,
@@ -33,5 +40,11 @@ static inline void stm32_gpio_pinctrl_bank_pin(struct pinctrl_state *p __unused,
 					       unsigned int *count __unused)
 {
 }
+
+static inline unsigned int __noreturn
+stm32_gpio_chip_bank_id(struct gpio_chip *chip __unused)
+{
+	panic();
+}
 #endif /*CFG_STM32_GPIO*/
 #endif /*__DRIVERS_STM32_GPIO_H*/
