@@ -12,6 +12,7 @@
 #include <dt-bindings/firewall/stm32mp13-tzc400.h>
 #include <dt-bindings/regulator/st,stm32mp13-regulator.h>
 #include <dt-bindings/reset/stm32mp13-resets.h>
+#include <dt-bindings/tamper/st,stm32mp13-tamp.h>
 #endif
 
 #ifdef CFG_STM32MP15
@@ -20,6 +21,7 @@
 #include <dt-bindings/firewall/stm32mp15-tzc400.h>
 #include <dt-bindings/regulator/st,stm32mp15-regulator.h>
 #include <dt-bindings/reset/stm32mp1-resets.h>
+#include <dt-bindings/tamper/st,stm32-tamp.h>
 #endif
 
 #ifdef CFG_STM32MP25
@@ -29,6 +31,17 @@
 #include <dt-bindings/firewall/stm32mp25-rifsc.h>
 #include <dt-bindings/firewall/stm32mp25-risaf.h>
 #include <dt-bindings/reset/st,stm32mp25-rcc.h>
+#include <dt-bindings/tamper/st,stm32mp25-tamp.h>
+#endif
+
+#ifdef CFG_STM32MP21
+#include <dt-bindings/clock/st,stm32mp21-rcc.h>
+#include <dt-bindings/clock/stm32mp21-clksrc.h>
+#include <dt-bindings/firewall/stm32mp25-rif.h>
+#include <dt-bindings/firewall/stm32mp21-rifsc.h>
+#include <dt-bindings/firewall/stm32mp25-risaf.h>
+#include <dt-bindings/reset/st,stm32mp21-rcc.h>
+#include <dt-bindings/tamper/st,stm32mp21-tamp.h>
 #endif
 
 #endif /* __DRIVERS_STM32MP_DT_BINDINGS_H */
