@@ -407,20 +407,21 @@ bool transfer_list_set_data_size(struct transfer_list_header *tl,
 		tl->size -= mov_dis;
 	}
 	/* Move all following entries to fit in the expanded or shrunk space */
-	memmove((void *)r_new_ev, (void *)old_ev, tl_old_ev - old_ev);
-
-	/*
-	 * Fill the gap due to round up/down with a void entry if the size of
-	 * the gap is more than an entry header.
-	 */
-	gap = r_new_ev - new_ev;
-	if (gap >= sizeof(*dummy_te)) {
-		/* Create a dummy transfer entry to fill up the gap */
-		dummy_te = (struct transfer_list_entry *)new_ev;
-		dummy_te->tag_id = TL_TAG_EMPTY;
-		dummy_te->reserved0 = 0;
-		dummy_te->hdr_size = sizeof(*dummy_te);
-		dummy_te->data_size = gap - sizeof(*dummy_te);
+	if (tl_old_ev > old_ev) {
+		memmove((void *)r_new_ev, (void *)old_ev, tl_old_ev - old_ev);
+		/*
+		 * Fill the gap due to round up/down with a void entry if the
+		 * size of the gap is more than an entry header.
+		 */
+		gap = r_new_ev - new_ev;
+		if (gap >= sizeof(*dummy_te)) {
+			/* Create a dummy transfer entry to fill up the gap */
+			dummy_te = (struct transfer_list_entry *)new_ev;
+			dummy_te->tag_id = TL_TAG_EMPTY;
+			dummy_te->reserved0 = 0;
+			dummy_te->hdr_size = sizeof(*dummy_te);
+			dummy_te->data_size = gap - sizeof(*dummy_te);
+		}
 	}
 
 	tl_e->data_size = new_data_size;
@@ -496,7 +497,7 @@ struct transfer_list_entry *transfer_list_add(struct transfer_list_header *tl,
 	tl->size += ev - tl_ev;
 
 	if (data)
-		memmove(tl_e + tl_e->hdr_size, data, data_size);
+		memmove(transfer_list_entry_data(tl_e), data, data_size);
 
 	transfer_list_update_checksum(tl);
 
