@@ -1846,7 +1846,7 @@ int mbedtls_mpi_exp_mod(mbedtls_mpi *X, const mbedtls_mpi *A,
                         mbedtls_mpi *prec_RR)
 {
 #if (defined(__KERNEL__) && defined(CFG_CORE_UNSAFE_MODEXP)) || \
-    (!defined(__KERNEL__) && defined(CFG_TA_MEBDTLS_UNSAFE_MODEXP))
+    (!defined(__KERNEL__) && defined(CFG_TA_MBEDTLS_UNSAFE_MODEXP))
     return mbedtls_mpi_exp_mod_unsafe(X, A, E, N, prec_RR);
 #else
     return mbedtls_mpi_exp_mod_optionally_safe(X, A, E, MBEDTLS_MPI_IS_SECRET, N, prec_RR);
