@@ -101,7 +101,7 @@ enum pkcs11_rc pkcs2tee_validate_rsa_pss(struct active_processing *proc,
 	if ((modulus_size % 8) == 1)
 		k = modulus_size / 8;
 	else
-		k = ROUNDUP(modulus_size, 8) / 8;
+		k = ROUNDUP_DIV(modulus_size, 8);
 
 	if (rsa_pss_ctx->salt_len > (k - 2 - hash_size))
 		return PKCS11_CKR_KEY_SIZE_RANGE;
@@ -569,6 +569,8 @@ enum pkcs11_rc load_tee_rsa_key_attrs(TEE_Attribute **tee_attrs,
 			break;
 		}
 
+		rc = PKCS11_CKR_GENERAL_ERROR;
+
 		if (pkcs2tee_load_attr(&attrs[count], TEE_ATTR_RSA_PRIME1, obj,
 				       PKCS11_CKA_PRIME_1))
 			count++;
@@ -775,3 +777,22 @@ size_t rsa_get_input_max_byte_size(TEE_OperationHandle op)
 
 	return info.maxKeySize / 8;
 }
+
+enum pkcs11_rc pkcs2tee_rsa_nopad_context(struct active_processing *proc)
+{
+	size_t key_size = 0;
+
+	/*
+	 * RSA no-pad (CKM_RSA_X_509) verify needs a buffer of the size
+	 * of the key to safely run.
+	 */
+	key_size = rsa_get_input_max_byte_size(proc->tee_op_handle);
+	if (!key_size)
+		return PKCS11_CKR_GENERAL_ERROR;
+
+	proc->extra_ctx = TEE_Malloc(key_size, TEE_USER_MEM_HINT_NO_FILL_ZERO);
+	if (!proc->extra_ctx)
+		return PKCS11_CKR_DEVICE_MEMORY;
+
+	return PKCS11_CKR_OK;
+}
